package com.esri.gwt.client;

import com.google.gwt.core.client.JavaScriptObject;

/**
 * The spatial reference of a map, layer, or inputs to and outputs
 * from a task. Each projected and geographic coordinate system is
 * defined by both a well-known ID (WKID) or a definition string (WKT).
 *  The JavaScript API supports only well-known ID's
 * 
 * @author Justin Merz
 */
public class SpatialReference extends JavaScriptObject {

	protected SpatialReference() { } 
	
	/**
	 * Creates a new SpatialReference object.
	 * 
	 * @param wkid - The well-known ID of a spatial reference
	 * @return SpatialReference
	 */
	public static native SpatialReference create(int wkid) /*-{
		return new $wnd.esri.SpatialReference({wkid:wkid});
	}-*/;
	
	/**
	 * The well-known ID of a spatial reference
	 * 
	 * @return int
	 */
	public final native int getWkid() /*-{ 
		if( this.wkid ) return this.wkid;
		return -1; 
	}-*/;

	public final native String getWkt() /*-{
		if( this.wkt ) return this.wkt;
		return "";
	}-*/;
	
}
