package com.esri.gwt.client.dijits;

import com.esri.gwt.client.Graphic;
import com.esri.gwt.client.OperationBase;
import com.esri.gwt.client.layers.FeatureLayer;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * The esri.dijit.editing namespace contains editing related opertions that inherit from OperationBase. 
 * These operations are used with the UndoManager to provide undo/redo functionality.
 * 
 * @author Justin Merz
 */
public class Delete extends OperationBase {
	
	protected Delete() {}
	
	/**
	 * Create a new Delete operation.
	 * 
	 * @param deletedGraphics - The features that were removed from the feature layer.
	 * @param featureLayer - The feature layer where the new feature(s) are added.
	 * @return Delete
	 */
	public static Delete create(Graphic[]  deletedGraphics, FeatureLayer featureLayer) {
		if( !assertLoaded() ) return JavaScriptObject.createObject().cast();
		return _create(deletedGraphics, featureLayer);
	}
	
	private static native Delete _create(Graphic[]  deletedGraphics, FeatureLayer featureLayer) /*-{
		return $wnd.esri.dijit.editing.editOperation.Delete({
			featureLayer: featureLayer,
			deletedGraphics: deletedGraphics
		});
	}-*/;
	
	/**
	 * Check that the esri.dijit.editing.editOperation has been loaded
	 * 
	 * @return boolean
	 */
	public static native boolean assertLoaded() /*-{
		if( $wnd.esri.dijit.editing.editOperation ) return true;
		$wnd.alert("The ESRI_DIJIT_EDITING_EDITOPERATION package has not been loaded");
		return false;
	}-*/;

}
