package com.esri.gwt.client.dijits;

import com.esri.gwt.client.TimeExtent;
import com.esri.gwt.client.Util;
import com.esri.gwt.client.event.TimeExtentChangeHandler;
import com.esri.gwt.client.layers.TimeInfo.TimeUnits;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.core.client.JsDate;
import com.google.gwt.event.logical.shared.AttachEvent;
import com.google.gwt.event.logical.shared.AttachEvent.Handler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.SimplePanel;

/**
 * Widget for visualizing time enabled layers.
 * 
 * @author Justin Merz
 */
public class TimeSlider extends Composite {
	
	private SimplePanel panel = new SimplePanel();
	private Parameters parameters;
	protected JavaScriptObject jso;
	
	/**
	 * Creates a new TimeSlider object.
	 * 
	 * @param params - Parameters for the time slider object. 
	 */
	public TimeSlider(Parameters params) {
		
		if( !assertLoaded() ) return; 
		
		parameters = params;
		
		initWidget(panel);
		
		addAttachHandler(new Handler(){
			@Override
			public void onAttachOrDetach(AttachEvent event) {
				if( jso == null ) jso = create(parameters);
			}
		});
		
	}
	
	private native JavaScriptObject create(Parameters params) /*-{
		return new $wnd.esri.dijit.TimeSlider(params, this.@com.esri.gwt.client.dijits.TimeSlider::getElement()());
	}-*/;
	
	/**
	 * Check that the esri.dijit.TimeSlider package has been loaded
	 * 
	 * @return boolean
	 */
	public static native boolean assertLoaded() /*-{
		if( $wnd.esri.dijit.TimeSlider ) return true;
		$wnd.alert("The ESRI_DIJIT_TIMESLIDER package has not been loaded");
		return false;
	}-*/;

	/**
	 * Default value is false.
	 * 
	 * @return boolean
	 */
	public native boolean isLoop() /*-{
		return this.@com.esri.gwt.client.dijits.TimeSlider::jso.loop;
	}-*/;
	
	/**
	 * Default value is false.
	 * 
	 * @return boolean
	 */
	public native boolean isPlaying() /*-{
		return this.@com.esri.gwt.client.dijits.TimeSlider::jso.playing;
	}-*/;
	
	/**
	 * Default value is 1.
	 * 
	 * @return int
	 */
	public native int getThumbCount() /*-{
		return this.@com.esri.gwt.client.dijits.TimeSlider::jso.thumbCount;
	}-*/;
	
	/**
	 * Rate at which the time animation plays. Default value is 1000 milliseconds.
	 * 
	 * @return int
	 */
	public native int getThumbMovingRate() /*-{
		return this.@com.esri.gwt.client.dijits.TimeSlider::jso.thumbMovingRate;
	}-*/;

	/**
	 * An array of dates representing the stops (tics) on the TimeSlider.
	 * 
	 * @return JsArray<JsDate>
	 */
	public native JsArray<JsDate> getTimeStops() /*-{
		return this.@com.esri.gwt.client.dijits.TimeSlider::jso.timeStops;
	}-*/;
	
	/**
	 * The specified number of time stops are created for the input time extent.
	 * 
	 * @param timeExtent - The time extent used to define the time slider's start and end time stops.
	 */
	public native void createTimeStopsByCount(TimeExtent timeExtent) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.createTimeStopsByCount(timeExtent);
	}-*/;
	
	/**
	 * The specified number of time stops are created for the input time extent.
	 * 
	 * @param timeExtent - The time extent used to define the time slider's start and end time stops.
	 * @param count - The number of time stops to create.
	 */
	public native void createTimeStopsByCount(TimeExtent timeExtent, int count) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.createTimeStopsByCount(timeExtent, count);
	}-*/;

	/**
	 * Create a time stop for each interval specified, i.e.(week,month, day).
	 * 
	 * @param timeExtent - The time extent used to define the time slider's start and end time stops.
	 */
	public native void createTimeStopsByInterval(TimeExtent timeExtent) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.createTimeStopsByInterval(timeExtent);
	}-*/;

	/**
	 * Create a time stop for each interval specified, i.e.(week,month, day).
	 * 
	 * @param timeExtent - The time extent used to define the time slider's start and end time stops.
	 * @param timeInterval - The length of the time interval.
	 * @param timeIntervalUnits - Time units.
	 */
	public void createTimeStopsByInterval(TimeExtent timeExtent, int timeInterval, TimeUnits timeIntervalUnits) {
		_createTimeStopsByInterval(timeExtent, timeInterval, timeIntervalUnits.getValue());
	}
	
	private native void _createTimeStopsByInterval(TimeExtent timeExtent, int timeInterval, String timeIntervalUnits) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.createTimeStopsByInterval(timeExtent, timeInterval, timeIntervalUnits);
	}-*/;
	
	/**
	 * Gets the current time extent for the time slider.
	 * 
	 * @return TimeExtent
	 */
	public native TimeExtent getCurrentTimeExtent() /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.getCurrentTimeExtent();
	}-*/;
	
	/**
	 * Move to the next time step.
	 */
	public native void next() /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.next();
	}-*/;
	
	/**
	 * Pause the time slider.
	 */
	public native void pause() /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.pause();
	}-*/;
	
	/**
	 * Play the time slider.
	 */
	public native void play() /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.play();
	}-*/;
	
	/**
	 * Move to the previous time step.
	 */
	public native void previous() /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.previous();
	}-*/;
	
	/**
	 * Specify an array of strings to be used as labels. The array position
	 * for each string defines which tick it will be placed on.
	 * 
	 * @param labels - An array of strings that define the labels for each tick.
	 */
	public void setLabels(String[] labels) {
		JavaScriptObject arr = JavaScriptObject.createArray();
		for( int i = 0; i < labels.length; i++ ) Util.addStringToArray( labels[i], arr );
		_setLabels(arr);
	};
	
	private native void _setLabels(JavaScriptObject array) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setLabels(array);
	}-*/;
	
	/**
	 * Determines whether or not loop. Default value is false.
	 * 
	 * @param loop - True plays the time slider continuously. Default value is false.
	 */
	public native void setLoop(boolean loop) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setLoop(loop);
	}-*/;
	
	/**
	 * The number of thumbs to display. A value of one displays a single thumb. Setting the
	 * value to 2 displays a range slider.
	 * 
	 * @param thumbCount - The number of thumbs to display. Default value is one.
	 */
	public native void setThumbCount(int thumbCount) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setThumbCount(thumbCount);
	}-*/;
	
	/**  
	 * 
	 * @param first - determinds where to put the first thumb.
	 * @param second - If it is a two thumb slider the second value determines where to place the additional thumb.
	 */
	public native void setThumbIndexes(int first, int second) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setThumbIndexes([first, second]);
	}-*/;
	
	/**
	 * Change the rate at which the time animation plays.
	 * 
	 * @param thumbMovingRate - The rate at which the time slider plays. Default value is 1500.
	 */
	public native void setThumbMovingRate(int thumbMovingRate) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setThumbMovingRate(thumbMovingRate);
	}-*/;
	
	/**
	 * Specify the number of ticks to display on the time slider.
	 * 
	 * @param tickCount - The number of ticks to display on the slider.
	 */
	public native void setTickCount(int tickCount) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setTickCount(tickCount);
	}-*/;
	
	/**
	 * Manually define the time stop locations by providing an array of dates.
	 * 
	 * @param stops - Array of dates
	 */
	public void setTimeStops(JsDate[] stops) {
		JavaScriptObject arr = JavaScriptObject.createObject();
		for( int i = 0; i < stops.length; i++ ) Util.addObjectToArray( stops[i], arr );
		_setTimeStops(arr);
	}
	
	private native void _setTimeStops(JavaScriptObject array) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.setTimeStops(array);
	}-*/;

	/**
	 * Determine if the time is displayed for an instant in time. Only valid when the thumb
	 * count is set to one.
	 * 
	 * @param createTimeInstants - When true, the time slider displays features for the
	 * current point in time. When false cumulative data is displayed from the start time
	 * to the current thumb location. The default value is false.
	 */
	public native void singleThumbAsTimeInstant(boolean createTimeInstants) /*-{
		this.@com.esri.gwt.client.dijits.TimeSlider::jso.singleThumbAsTimeInstant(createTimeInstants);
	}-*/;
	
	/**
	 * Add time extent changed handler.
	 * 
	 * @param handler - Fires when the timeExtent of the TimeSlider is changed.
	 */
	public final native void addTimeExtentChangeHandler(TimeExtentChangeHandler handler ) /*-{
		$wnd.dojo.connect(
		 	this.@com.esri.gwt.client.dijits.TimeSlider::jso,
		 	"onTimeExtentChange",
		 	function(timeExtent) {
		 		handler.@com.esri.gwt.client.event.TimeExtentChangeHandler::onTimeExtentChange(Lcom/esri/gwt/client/TimeExtent;)(timeExtent);
			}
		);
	}-*/;
	
	public static class Parameters extends JavaScriptObject {
	
		protected Parameters() {}
		
		public static Parameters create() {
			return JavaScriptObject.createObject().cast();
		}
		
		/**
		 * 
		 * @param excludeDataAtLeadingThumb - When true, subtracts one second to the time extent's end time to exclude data at
		 * the exact end time instant.
		 */
		public native final void setExcludeDataAtLeadingThumb(boolean excludeDataAtLeadingThumb) /*-{
			this["excludeDataAtLeadingThumb"] = excludeDataAtLeadingThumb;
		}-*/;
		
		/**
		 * 
		 * @param excludeDataAtTrailingThumb - When true, adds one second to the time extent's start time to exclude data at
		 * the exact start time instant.
		 */
		public native final void setExcludeDataAtTrailingThumb(boolean excludeDataAtTrailingThumb) /*-{
			this["excludeDataAtTrailingThumb"] = excludeDataAtTrailingThumb;
		}-*/;
		
	}
	
}
