package com.esri.gwt.client.geometry;

import com.esri.gwt.client.SpatialReference;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayNumber;

/**
 *  A location defined by an X- and Y- coordinate. It can be map units or screen units.
 *  
 * @author lsimone
 * @author Justin Merz
 */
public class Point extends Geometry {

	protected Point () {}
	
	/**
	 * Creates a new Point object using x, y, and a spatial reference.
	 * 
	 * @param x - X-coordinate of a point in map units. 
	 * @param y - Y-coordinate of a point in map units. 
	 * @param sr - Spatial reference of the geometry.
	 * @return Point
	 */
	public native static Point create(float x, float y, SpatialReference sr) /*-{
		return new $wnd.esri.geometry.Point(x, y, sr);
	}-*/;
	
	public native static Point create(double x, double y, SpatialReference sr) /*-{
		return new $wnd.esri.geometry.Point(x, y, sr);
	}-*/;
	
	/**
	 * Creates a new Point object using an array containing an x,y coordinate value and a spatial reference.
	 * 
	 * @param xy - An array that includes an x,y coordinate.
	 * @param sr - Spatial reference of the geometry. 
	 * @return Point
	 */
	public native static Point create(JsArrayNumber xy, SpatialReference sr) /*-{
		return new $wnd.esri.geometry.Point(xy, sr);
	}-*/;
	
	/**
	 * Creates a new Point object using a JSON object.
	 * 
	 * @param json - A JSON object that contains an x,y coordinate. 
	 * @return Point
	 */
	public native static Point create(JavaScriptObject json) /*-{
		return new $wnd.esri.geometry.Point(json);
	}-*/;
	
	/**
	 * X-coordinate of a point in map units.
	 * 
	 * @return float
	 */
	public final native float getX() /*-{ 
		return this.x; 
	}-*/;
	
	/**
	 * Y-coordinate of a point in map units.
	 * 
	 * @return float
	 */
	public final native float getY() /*-{ 
		return this.y; 
	}-*/;
	
	/**
	 * Offsets the point in an x and y direction. Units are map units.
	 * 
	 * @param dx - Value for x-coordinate of point.
	 * @param dy - Value for y-coordinate of point.
	 * @return Point
	 */
	public final native Point getOffset(float dx, float dy) /*-{ 
		return this.offset(dx, dy) 
	}-*/;
	
	/**
	 * Sets x-coordinate of point. 
	 * 
	 * @param x - Value for x-coordinate of point.
	 * @return Point
	 */
	public final native Point setX(float x) /*-{ 
		this.setX(x); 
	}-*/;
	
	/**
	 * Sets y-coordinate of point. 
	 * 
	 * @param y - Value for y-coordinate of point.
	 * @return Point
	 */
	public final native Point setY(float y) /*-{ 
		return this.setY(y); 
	}-*/;
	
	/**
	 * Updates a point.
	 * 
	 * @param dx - X-coordinate of the updated point.
	 * @param dy - Y-coordinate of the updated point.
	 * @return Point
	 */
	public final native Point update(float dx, float dy) /*-{ 
		return this.update(dx, dy) 
	}-*/;

	/**
	 * Return the point as a formated string
	 * 
	 * @return String
	 */
	public final String toNiceString() {
		return "[" + getX() + "," + getY() + "]";
	}
	
}
