package com.esri.gwt.client.layers;

import com.esri.gwt.client.TimeExtent;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * Time information details. 
 * 
 * @author Justin Merz
 */
public class TimeInfo extends JavaScriptObject {
	
	public static enum TimeUnits {
		NOT_SET(""),
		UNIT_CENTURIES("esritTimeUnitsCenturies"),
		UNIT_DAYS("esritTimeUnitsDays"),
		UNIT_DECADES("esritTimeUnitsDecades"),
		UNIT_HOURS("esritTimeUnitsHours"),
		UNIT_MILLISECONDS("esritTimeUnitsMilliseconds"),
		UNIT_MINUTES("esritTimeUnitsMinutes"),
		UNIT_MONTHS("esritTimeUnitsMonths"),
		UNIT_SECONDS("esritTimeUnitsSeconds"),
		UNIT_UNKNOWN("esritTimeUnitsUnknown"),
		UNIT_WEEKS("esritTimeUnitsWeeks"),
		UNIT_YEARS("esritTimeUnitsYears");
		private String val;
		private TimeUnits(String value){
			val = value;
		}
		public String getValue(){
			return val;
		}
	}
	
	protected TimeInfo() {}
	
	/**
	 * The name of the attribute field that contains the end time information.
	 * 
	 * @return String
	 */
	public final native String getEndTimeField() /*-{
		return this.endTimeField;
	}-*/;
	
	/**
	 * Default time-related export options for the layer. When using a dynamic
	 * map service, these options can be overriden for sub-layers using the setLayerTimeOptions method.
	 * 
	 * @return LayerTimeOptions
	 */
	public final native LayerTimeOptions getExportOptions() /*-{
		return this.exportOptions;
	 }-*/;
	
	/**
	 * The name of the attribute field that contains the start time information.
	 * 
	 * @return String
	 */
	public final native String getStartTimeField() /*-{
		return this.startTimeField;
	}-*/;
	
	/**
	 * The time extent for all the data in the layer.
	 * 
	 * @return TimeExtent
	 */
	public final native TimeExtent getTimeExtent() /*-{
		return this.timeExtent;
	}-*/;
	
	/**
	 * Time interval of the data in the layer. Typically used for the TimeSlider when animating the layer.
	 * 
	 * @return int
	 */
	public final native int getTimeInterval() /*-{
		return this.timeInterval;
	}-*/;
	
	/**
	 * Temporal unit in which the time interval is measured. See the Constants table for valid values.
	 * 
	 * @return String
	 */
	public final native String getTimeIntervalUnits() /*-{
		return this.timeIntervalUnits;
	}-*/;
	
	/**
	 * Information about how the time was measured.
	 * 
	 * @return TimeReference;
	 */
	public final native TimeReference getTimeReference() /*-{
		return this.timeReference;
	}-*/;

	/**
	 * The field that contains the trackId.
	 * 
	 * @return - String
	 */
	public final native String getTrackIdField() /*-{
		return this.trackIdField;
	}-*/;

}
