package com.esri.gwt.client.renderer;

import com.esri.gwt.client.Util;
import com.esri.gwt.client.dojo.Color;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * Time class breaks ager displays aging by classifying features based on an age range.
 * 
 * @author Justin Merz
 */
public class TimeClassBreaksAger extends SymbolAger {
	
	protected TimeClassBreaksAger() {}
	
	public enum TimeUnits {
		UNIT_DAYS("day"),
		UNIT_HOURS("hour"),
		UNIT_MILLISECONDS("millisecond"),
		UNIT_MINUTES("minute"),
		UNIT_MONTHS("month"),
		UNIT_SECONDS("second"),
		UNIT_WEEKS("week"),
		UNIT_YEARS("year");
		private String value = "";
		TimeUnits(String val) {
			value = val;
		}
		public String getValue() {
			return value;
		}
	}
	
	/**
	 * Creates a new TimeClassBreaksAgerObject with the specified time breaks inforamtion.
	 * 
	 * @param infos - Each element in the array is an object that describes the class breaks information.
	 * @return TimeClassBreaksAger
	 */
	public static TimeClassBreaksAger create(Info[] infos) {
		JavaScriptObject array = JavaScriptObject.createArray();
		for( int i = 0; i < infos.length; i++ ) Util.addObjectToArray(infos[i], array);
		return _create(array);
	}
	
	private static native TimeClassBreaksAger _create(JavaScriptObject infos) /*-{
		return new $wnd.esri.renderer.TimeClassBreaksAger(infos);
	}-*/;

	/**
	 * Creates a new TimeClassBreaksAgerObject with the specified time breaks inforamtion.
	 * 
	 * @param infos - Each element in the array is an object that describes the class breaks information.
	 * @param timeUnits - The unit in which the minimum and maximum break values are measured.  Default is days.
	 * @return TimeClassBreaksAger
	 */
	public static TimeClassBreaksAger create(Info[] infos, TimeUnits timeUnits) {
		JavaScriptObject array = JavaScriptObject.createArray();
		for( int i = 0; i < infos.length; i++ ) Util.addObjectToArray(infos[i], array);
		return _create(array, timeUnits.getValue());
	}
	
	private static native TimeClassBreaksAger _create(JavaScriptObject infos, String timeUnits) /*-{
		return new $wnd.esri.renderer.TimeClassBreaksAger(infos, timeUnits);
	}-*/;
	
	public static class Info extends JavaScriptObject {
		
		protected Info() {}
		
		/**
		 * object that describes the class breaks information.
		 * 
		 * @param minAge - The minimum age for the break info.
		 * @param maxAge - The maximum age for the break info.
		 * @param color - The color for the break.
		 * @param size - The size for the break.
		 * @return Info
		 */
		public static native Info create(int minAge, int maxAge, Color color, int size) /*-{
			return { minAge: minAge, maxAge: maxAge, color: color, size: size };
		}-*/;
		
		/**
		 * object that describes the class breaks information.
		 * 
		 * @param minAge - The minimum age for the break info.
		 * @param maxAge - The maximum age for the break info.
		 * @param size - The size for the break.
		 * @return Info
		 */
		public static native Info create(int minAge, int maxAge, int size) /*-{
			return { minAge: minAge, maxAge: maxAge, size: size };
		}-*/;
		
		/**
		 * object that describes the class breaks information.
		 * 
		 * @param minAge - The minimum age for the break info.
		 * @param maxAge - The maximum age for the break info.
		 * @param color - The color for the break.
		 * @return Info
		 */
		public static native Info create(int minAge, int maxAge, Color color ) /*-{
			return { minAge: minAge, maxAge: maxAge, color: color };
		}-*/;
		
		public final native void setMaxAgeInfinity() /*-{
			this.maxAge = "Infinity";
		}-*/;
		
	}
	
}
