package com.esri.gwt.client.tasks;

import com.esri.gwt.client.SpatialReference;
import com.esri.gwt.client.Util;
import com.esri.gwt.client.callback.AreasAndLengthsCallback;
import com.esri.gwt.client.callback.AutoCompleteCallback;
import com.esri.gwt.client.callback.BufferCallback;
import com.esri.gwt.client.callback.ConvexHullCallback;
import com.esri.gwt.client.callback.CutCallback;
import com.esri.gwt.client.callback.DifferenceCallback;
import com.esri.gwt.client.callback.DistanceCallback;
import com.esri.gwt.client.callback.GeneralizeCallback;
import com.esri.gwt.client.callback.IntersectCallback;
import com.esri.gwt.client.callback.LabelPointsCallback;
import com.esri.gwt.client.callback.LengthsCallback;
import com.esri.gwt.client.callback.OffsetCallback;
import com.esri.gwt.client.callback.ProjectCallback;
import com.esri.gwt.client.callback.RelationCallback;
import com.esri.gwt.client.callback.ReshapeCallback;
import com.esri.gwt.client.callback.SimplifyCallback;
import com.esri.gwt.client.callback.TrimExtendCallback;
import com.esri.gwt.client.callback.UnionCallback;
import com.esri.gwt.client.dojo.Deferred;
import com.esri.gwt.client.event.AreasAndLengthsHandler;
import com.esri.gwt.client.event.AutoCompleteHandler;
import com.esri.gwt.client.event.BufferHandler;
import com.esri.gwt.client.event.ConvexHullHandler;
import com.esri.gwt.client.event.CutHandler;
import com.esri.gwt.client.event.DifferenceHandler;
import com.esri.gwt.client.event.DistanceHandler;
import com.esri.gwt.client.event.ErrorHandler;
import com.esri.gwt.client.event.GeneralizeHandler;
import com.esri.gwt.client.event.IntersectHandler;
import com.esri.gwt.client.event.LabelPointsHandler;
import com.esri.gwt.client.event.LengthsHandler;
import com.esri.gwt.client.event.OffsetHandler;
import com.esri.gwt.client.event.ProjectHandler;
import com.esri.gwt.client.event.RelationHandler;
import com.esri.gwt.client.event.ReshapeHandler;
import com.esri.gwt.client.event.SimplifyHandler;
import com.esri.gwt.client.event.UnionHandler;
import com.esri.gwt.client.geometry.Geometry;
import com.esri.gwt.client.geometry.Polygon;
import com.esri.gwt.client.geometry.Polyline;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;

/**
 * Represents a geometry service resource exposed by the ArcGIS Server REST API. It is used to perform various
 * operations on geometries such as project, simplify, buffer, and relationships.
 * 
 * @author Justin Merz
 */
public class GeometryService extends JavaScriptObject {
	
	public enum UnitType {
		NOT_SET(""),
		UNIT_150KILOMETER_LENGTH("109031"),
		UNIT_50KILOMETER_LENGTH("109030"),
		UNIT_ACRES	("esriAcres"),
		UNIT_ARCMINUTE("9103"),
		UNIT_ARCMINUTE_CENTESIMAL("9112"),
		UNIT_ARCSECOND("9104"),
		UNIT_ARCSECOND_CENTESIMAL("9113"),
		UNIT_ARES("esriAres"),
		UNIT_BENOIT_1895A_CHAIN("9052"),
		UNIT_BENOIT_1895A_FOOT("9051"),
		UNIT_BENOIT_1895A_LINK("9053"),
		UNIT_BENOIT_1895A_YARD("9050"),
		UNIT_BENOIT_1895B_CHAIN("9062"),
		UNIT_BENOIT_1895B_FOOT("9061"),
		UNIT_BENOIT_1895B_LINK("9063"),
		UNIT_BENOIT_1895B_YARD("9060"),
		UNIT_BRITISH_1936_FOOT("9095"),
		UNIT_CENTIMETER("109006"),
		UNIT_CLARKE_CHAIN("9038"),
		UNIT_CLARKE_FOOT("9005"),
		UNIT_CLARKE_LINK("9039"),
		UNIT_CLARKE_YARD("9037"),
		UNIT_DECIMETER("109005"),
		UNIT_DEGREE("9102"),
		UNIT_FATHOM("9014"),
		UNIT_FOOT("9002"),
		UNIT_FOOT_1865("9070"),
		UNIT_GERMAN_METER("9031"),
		UNIT_GOLDCOAST_FOOT("9094"),
		UNIT_GON("9106"),
		UNIT_GRAD("9105"),
		UNIT_HECTARES("esriHectares"),
		UNIT_INDIAN_1937_FOOT("9081"),
		UNIT_INDIAN_1937_YARD("9085"),
		UNIT_INDIAN_1962_FOOT("9082"),
		UNIT_INDIAN_1962_YARD("9086"),
		UNIT_INDIAN_1975_FOOT("9083"),
		UNIT_INDIAN_1975_YARD("9087"),
		UNIT_INDIAN_FOOT("9080"),
		UNIT_INDIAN_YARD("9084"),
		UNIT_INTERNATIONAL_CHAIN("109003"),
		UNIT_INTERNATIONAL_INCH("109008"),
		UNIT_INTERNATIONAL_LINK("109004"),
		UNIT_INTERNATIONAL_ROD("109010"),
		UNIT_INTERNATIONAL_YARD("109001"),
		UNIT_KILOMETER("9036"),
		UNIT_METER("9001"),
		UNIT_MICRORADIAN("9109"),
		UNIT_MIL6400("9114"),
		UNIT_MILLIMETER("109007"),
		UNIT_NAUTICAL_MILE("9030"),
		UNIT_RADIAN("9103"),
		UNIT_SEARS_CHAIN("9042"),
		UNIT_SEARS_FOOT("9041"),
		UNIT_SEARS_LINK("9043"),
		UNIT_SEARS_YARD("9040"),
		UNIT_SQUARE_CENTIMETERS("esriSquareCentimeters"),
		UNIT_SQUARE_DECIMETERS("esriSquareDecimeters"),
		UNIT_SQUARE_FEET("esriSquareFeet"),
		UNIT_SQUARE_INCHES("esriSquareInches"),
		UNIT_SQUARE_KILOMETERS("esriSquareKilometers"),
		UNIT_SQUARE_METERS("esriSquareMeters"),
		UNIT_SQUARE_MILES("esriSquareMiles"),
		UNIT_SQUARE_MILLIMETERS("esriSquareMillimeters"),
		UNIT_SQUARE_YARDS("esriSquareYards"),
		UNIT_STATUTE_MILE("9093"),
		UNIT_SURVEY_CHAIN("9033"),
		UNIT_SURVEY_FOOT("9003"),
		UNIT_SURVEY_LINK("9034"),
		UNIT_SURVEY_MILE("9035"),
		UNIT_SURVEY_YARD("109002"),
		UNIT_UK_NAUTICAL_MILE("109013"),
		UNIT_US_NAUTICAL_MILE("109012"),
		UNIT_US_SURVEY_INCH("109009"),
		UNIT_US_SURVEY_ROD("109011");
		
		private String val = "";
		UnitType(String value) {
			val = value;
		}
		public String getValue(){
			return val;
		}
	}
	
	protected GeometryService() {}
	
	/**
	 * Creates a new GeometryService object.
	 * 
	 * @param url - URL to the ArcGIS Server REST resource that represents a geometry service.
	 * @return GeometryService
	 */
	public static native GeometryService create(String url) /*-{
		return new $wnd.esri.tasks.GeometryService(url);
	}-*/;

	/**
	 * URL to the ArcGIS Server REST resource that represents a locator service. 
	 * 
	 * @return String
	 */
	public final native String getUrl() /*-{
		return this.url;
	}-*/;
	
	/**
	 * Computes the area and length for the input polygons.
	 * 
	 * @param param - Specify the input polygons and optionally the linear and areal units.
	 * @param callback - The function to call when the method has completed
	 * @return Deferred
	 */
	public final native Deferred areasAndLengths(AreasAndLengthsParameters param, AreasAndLengthsCallback callback) /*-{		
		return this.areasAndLengths(param, 
			function(results) {
				callback.@com.esri.gwt.client.callback.AreasAndLengthsCallback::onAreasAndLengthsComplete(Lcom/esri/gwt/client/event/AreasAndLengthsHandler$AreasAndLengths;)(results);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.AreasAndLengthsCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Computes the area and length for the input polygons.
	 * 
	 * @param param - Specify the input polygons and optionally the linear and areal units.
	 * @return Deferred
	 */
	public final native Deferred areasAndLengths(AreasAndLengthsParameters param) /*-{
		return this.areasAndLengths(param);
	}-*/;
	
	/**
	 * The Auto Complete operation is performed on a geometry service resource. The AutoComplete operation simplifies the process of constructing
	 * new polygons that are adjacent to other polygons. It constructs polygons that fill in the gaps between existing polygons and a set of polylines.
	 * 
	 * @param polygons - The array of polygons that will provide some boundaries for new polygons.
	 * @param polylines - An array of polylines that will provide the remaining boundaries for new polygons.
	 * @param callback - The function to call when the method has completed. 
	 * @return Deferred
	 */
	public final Deferred autoComplete(Polygon[] polygons, Polyline[] polylines, AutoCompleteCallback callback) {
		JavaScriptObject arr1 = JavaScriptObject.createArray().cast();
		JavaScriptObject arr2 = JavaScriptObject.createArray().cast();
		for( int i = 0; i < polygons.length; i++ ) Util.addObjectToArray(polygons[i], arr1);
		for( int i = 0; i < polylines.length; i++ ) Util.addObjectToArray(polylines[i], arr2);
		return _autoComplete(arr1, arr2, callback);
	}
	
	private final native Deferred _autoComplete(JavaScriptObject polygons, JavaScriptObject polylines, AutoCompleteCallback callback) /*-{		
		return this.autoComplete(polygons, polylines, 
			function(polygons) {
				callback.@com.esri.gwt.client.callback.AutoCompleteCallback::onAutoCompleteComplete(Lcom/google/gwt/core/client/JsArray;)(polygons);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.AutoCompleteCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The Auto Complete operation is performed on a geometry service resource. The AutoComplete operation simplifies the process of constructing
	 * new polygons that are adjacent to other polygons. It constructs polygons that fill in the gaps between existing polygons and a set of polylines.
	 * 
	 * @param polygons - The array of polygons that will provide some boundaries for new polygons.
	 * @param polylines - An array of polylines that will provide the remaining boundaries for new polygons.
	 * @return Deferred
	 */
	public final Deferred autoComplete(Polygon[] polygons, Polyline[] polylines) {
		JavaScriptObject arr1 = JavaScriptObject.createArray().cast();
		JavaScriptObject arr2 = JavaScriptObject.createArray().cast();
		for( int i = 0; i < polygons.length; i++ ) Util.addObjectToArray(polygons[i], arr1);
		for( int i = 0; i < polylines.length; i++ ) Util.addObjectToArray(polylines[i], arr2);
		return _autoComplete(arr1, arr2);
	}
	
	private final native Deferred _autoComplete(JavaScriptObject polygons, JavaScriptObject polylines) /*-{		
		return this.autoComplete(polygons, polylines);
	}-*/;
	
	/**
	 * Creates buffer polygons at a specified distance around the given geometries. On completion, the onBufferComplete event is fired and the optional
	 * callback function is invoked. Both the callback and event handlers receive an array of Geometry that contains the buffer polygons.
	 * 
	 * @param params - Specifies the input geometries, buffer distances, and other options.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred buffer(BufferParameters params, BufferCallback callback) /*-{	
		return this.buffer(params, 
			function(geometries) {
				callback.@com.esri.gwt.client.callback.BufferCallback::onBufferComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.BufferCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Creates buffer polygons at a specified distance around the given geometries. On completion, the onBufferComplete event is fired and the optional
	 * callback function is invoked. Both the callback and event handlers receive an array of Geometry that contains the buffer polygons.
	 * 
	 * @param params - Specifies the input geometries, buffer distances, and other options.
	 * @return Deferred
	 */
	public final native Deferred buffer(BufferParameters params) /*-{
		return this.buffer(params);
	}-*/;

	/**
	 * The convexHull operation is performed on a geometry service resource. It returns the convex hull of the input geometry. The input geometry can
	 * be a point, multipoint, polyline or polygon. The hull is typically a polygon but can also be a polyline or point in degenerate cases.
	 * 
	 * @param geometries - The geometries whose convex hull is to be created.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred convexHull(Geometry[] geometries, ConvexHullCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _convexHull(arr, callback);
	}
	
	private final native Deferred _convexHull(JavaScriptObject geometries, ConvexHullCallback callback) /*-{		
		return this.convexHull(geometries, 
			function(geometry) {
				callback.@com.esri.gwt.client.callback.ConvexHullCallback::onConvexHullComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.ConvexHullCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The convexHull operation is performed on a geometry service resource. It returns the convex hull of the input geometry. The input geometry can
	 * be a point, multipoint, polyline or polygon. The hull is typically a polygon but can also be a polyline or point in degenerate cases.
	 * 
	 * @param geometries - The geometries whose convex hull is to be created.
	 * @return Deferred
	 */
	public final Deferred convexHull(Geometry[] geometries) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _convexHull(arr);
	}
	
	private final native Deferred _convexHull(JavaScriptObject geometries) /*-{		
		return this.convexHull(geometries);
	}-*/;
	
	/**
	 * The cut operation is performed on a geometry service resource. This operation splits the input polyline or polygon where it crosses a cutting polyline. 
	 * 
	 * @param targetGeometry - The polyline or polygon to be cut.
	 * @param cutterGeometry - The polyline that will be used to divide the target into pieces where it crosses the target.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred cut(Geometry[] targetGeometry, Geometry[] cutterGeometry, CutCallback callback) {
		JavaScriptObject arr1 = JavaScriptObject.createArray().cast();
		JavaScriptObject arr2 = JavaScriptObject.createArray().cast();
		for( int i = 0; i < targetGeometry.length; i++ ) Util.addObjectToArray(targetGeometry[i], arr1);
		for( int i = 0; i < cutterGeometry.length; i++ ) Util.addObjectToArray(cutterGeometry[i], arr2);
		return _cut(arr1, arr2, callback);
	}
	
	private final native Deferred _cut(JavaScriptObject targetGeometry, JavaScriptObject cutterGeometry, CutCallback callback) /*-{		
		return this.cut(targetGeometry, cutterGeometry, 
			function(geometries) {
				callback.@com.esri.gwt.client.callback.CutCallback::onCutComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.CutCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;

	/**
	 * The cut operation is performed on a geometry service resource. This operation splits the input polyline or polygon where it crosses a cutting polyline. 
	 * 
	 * @param targetGeometry - The polyline or polygon to be cut.
	 * @param cutterGeometry - The polyline that will be used to divide the target into pieces where it crosses the target.
	 * @return Deferred
	 */
	public final Deferred cut(Geometry[] targetGeometry, Geometry[] cutterGeometry) {
		JavaScriptObject arr1 = JavaScriptObject.createArray().cast();
		JavaScriptObject arr2 = JavaScriptObject.createArray().cast();
		for( int i = 0; i < targetGeometry.length; i++ ) Util.addObjectToArray(targetGeometry[i], arr1);
		for( int i = 0; i < cutterGeometry.length; i++ ) Util.addObjectToArray(cutterGeometry[i], arr2);
		return _cut(arr1, arr2);
	}
	
	private final native Deferred _cut(JavaScriptObject targetGeometry, JavaScriptObject cutterGeometry) /*-{		
		return this.cut(targetGeometry, cutterGeometry);
	}-*/;
	
	/**
	 * The difference operation is performed on a geometry service resource. This operation constructs the set-theoretic difference
	 * between an array of geometries and another geometry.
	 * 
	 * @param geometries - An array of points, multipoints, polylines or polygons.
	 * @param geometry - A single geometry of any type, of dimension equal to or greater than the elements of geometries.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred difference(Geometry[] geometries, Geometry geometry, DifferenceCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _difference(arr, geometry, callback);
	}
	
	private final native Deferred _difference(JavaScriptObject geometries, Geometry geometry, DifferenceCallback callback) /*-{		
		return this.difference(geometries, geometry, 
			function(geometries) {
				callback.@com.esri.gwt.client.callback.DifferenceCallback::onDifferenceComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.DifferenceCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The difference operation is performed on a geometry service resource. This operation constructs the set-theoretic difference
	 * between an array of geometries and another geometry.
	 * 
	 * @param geometries - An array of points, multipoints, polylines or polygons.
	 * @param geometry - A single geometry of any type, of dimension equal to or greater than the elements of geometries.
	 * @return Deferred
	 */
	public final Deferred difference(Geometry[] geometries, Geometry geometry) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _difference(arr, geometry);
	}
	
	private final native Deferred _difference(JavaScriptObject geometries, Geometry geometry) /*-{		
		return this.difference(geometries, geometry);
	}-*/;
	
	/**
	 * Measures the planar or geodesic distance between geometries. 
	 * 
	 * @param params - Sets the input geometries to measure, distance units and other parameters.
	 * @param callback - The function to call when the method has completed. 
	 * @return - Deferred
	 */
	public final native Deferred distance(DistanceParameters params, DistanceCallback callback) /*-{
		return this.distance(distance, 
			function(distance) {
				callback.@com.esri.gwt.client.callback.DistanceCallback::onDistanceComplete(F)(distance);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.DistanceCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Measures the planar or geodesic distance between geometries. 
	 * 
	 * @param params - Sets the input geometries to measure, distance units and other parameters.
	 * @return - Deferred
	 */
	public final native Deferred distance(DistanceParameters params) /*-{
		return this.distance(distance);
	}-*/;
	
	/**
	 * Generalizes the input geomtries using the Douglas-Peucker algorithim.
	 * 
	 * @param params - An array of geometries to generalize and a maximum deviation.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred generalize(GeneralizeParameters params, GeneralizeCallback callback) /*-{		
		return this.generalize(params, 
			function(geometries) {
				callback.@com.esri.gwt.client.callback.GeneralizeCallback::onGeneralizeComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.GeneralizeCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Generalizes the input geomtries using the Douglas-Peucker algorithim.
	 * 
	 * @param params - An array of geometries to generalize and a maximum deviation.
	 * @return Deferred
	 */
	public final native Deferred generalize(GeneralizeParameters params) /*-{		
		return this.generalize(params);
	}-*/;
	
	/**
	 * The intersect operation is performed on a geometry service resource. This operation constructs the set-theoretic intersection
	 * between an array of geometries and another geometry.
	 * 
	 * @param geometries - An array of points, multipoints, polylines or polygons.
	 * @param geometry -  single geometry of any type, of dimension equal to or greater than the elements of geometries.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred intersect(Geometry[] geometries, Geometry geometry, IntersectCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _intersect(arr, geometry, callback);
	}
	
	private final native Deferred _intersect(JavaScriptObject geometries, Geometry geometry, IntersectCallback callback) /*-{		
		return this.intersect(geometries, geometry, 
			function(geometries) {
				callback.@com.esri.gwt.client.callback.IntersectCallback::onIntersectComplete(Lcom/google/gwt/core/client/JsArray;)(intersect);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.IntersectCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The intersect operation is performed on a geometry service resource. This operation constructs the set-theoretic intersection
	 * between an array of geometries and another geometry.
	 * 
	 * @param geometries - An array of points, multipoints, polylines or polygons.
	 * @param geometry -  single geometry of any type, of dimension equal to or greater than the elements of geometries.
	 * @return Deferred
	 */
	public final Deferred intersect(Geometry[] geometries, Geometry geometry) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _intersect(arr, geometry);
	}
	
	private final native Deferred _intersect(JavaScriptObject geometries, Geometry geometry) /*-{		
		return this.intersect(geometries, geometry);
	}-*/;
	
	/**
	 * Calculates an interior point for each polygon specified. These interior points can be used by clients for labeling the polygons.
	 * 
	 * @param polygons - The graphics to process.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred labelPoints(JsArray<Geometry> polygons, LabelPointsCallback callback) /*-{		
		return this.labelPoints(polygons, 
			function(labelPoints) {
				callback.@com.esri.gwt.client.callback.LabelPointsCallback::onLabelPointsComplete(Lcom/google/gwt/core/client/JsArray;)(labelPoints);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.LabelPointsCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Calculates an interior point for each polygon specified. These interior points can be used by clients for labeling the polygons.
	 * 
	 * @param polygons - The graphics to process.
	 * @return Deferred
	 */
	public final native Deferred labelPoints(JsArray<Geometry> polygons) /*-{
		return this.labelPoints(polygons);
	}-*/;
	
	/**
	 * Gets the lengths for a Geometry[] when the geometry type is Polyline.
	 * 
	 * @param params - Specify the polylines and optionally the length unit and the geodesic length option.
	 * @param callback - The function to call when the method has completed. 
	 * @return Deferred
	 */
	public final native Deferred lengths(LengthsParameters params, LengthsCallback callback) /*-{		
		return this.lengths(params, 
			function(lengths) {
				callback.@com.esri.gwt.client.callback.LengthsCallback::onLengthsComplete(Lcom/esri/gwt/client/event/LengthsHandler$Lengths;)(lengths);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.LengthsCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Gets the lengths for a Geometry[] when the geometry type is Polyline.
	 * 
	 * @param params - Specify the polylines and optionally the length unit and the geodesic length option.
	 * @return Deferred
	 */
	public final native Deferred lengths(LengthsParameters params) /*-{
		return this.lengths(params);
	}-*/;

	/**
	 * Constructs the offset of the input geometries. If the offsetDistance is positive the constructed offset will be on the right side of the geometry. Left
	 * side offsets are constructed with negative values.
	 * 
	 * @param params - Set the geometries to offset, distance and units.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred offset(OffsetParameters params, OffsetCallback callback) /*-{
		return this.offset(params,
			function(geometries) {
				callback.@com.esri.gwt.client.callback.OffsetCallback::onOffsetComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.OffsetCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Constructs the offset of the input geometries. If the offsetDistance is positive the constructed offset will be on the right side of the geometry. Left
	 * side offsets are constructed with negative values.
	 * 
	 * @param params - Set the geometries to offset, distance and units.
	 * @return Deferred
	 */
	public final native Deferred offset(OffsetParameters params) /*-{
		return this.offset(params);
	}-*/;
	
	/**
	 * Projects a set of geometries into a new spatial reference. On completion, the onProjectComplete event is fired and the optional callback function is
	 * invoked. Both the callback and event handlers receive an array of Geometry[] that contains the projected geometries.
	 * 
	 * @param geometries - The geometries to project.
	 * @param spatialReference - The well-known ID of a spatial reference. 
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred project(JsArray<Geometry> geometries, SpatialReference spatialReference, ProjectCallback callback) /*-{		
		return this.project(geometries, spatialReference,
			function(geometries) {
				callback.@com.esri.gwt.client.callback.ProjectCallback::onProjectComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.ProjectCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Projects a set of geometries into a new spatial reference. On completion, the onProjectComplete event is fired and the optional callback function is
	 * invoked. Both the callback and event handlers receive an array of Geometry[] that contains the projected geometries.
	 * 
	 * @param geometries - The geometries to project.
	 * @param spatialReference - The well-known ID of a spatial reference. 
	 * @return Deferred
	 */
	public final native Deferred project(JsArray<Geometry> geometries, SpatialReference spatialReference) /*-{		
		return this.project(geometries, spatialReference);
	}-*/;
	
	/**
	 * Computes the set of pairs of geometries from the input geometry arrays that belong to the specified relation. Both arrays
	 * are assumed to be in the same spatial reference. The relations are evaluated in 2D. Z coordinates are not used. Geometry
	 * types cannot be mixed within an array.
	 * 
	 * @param params - The set of parameters required to perform the comparison.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred relation(RelationParameters params, RelationCallback callback) /*-{		
		return this.relation(params,
			function(relationships) {
				callback.@com.esri.gwt.client.callback.RelationCallback::onRelationComplete(Lcom/google/gwt/core/client/JsArray;)(relationships);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.RelationCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Computes the set of pairs of geometries from the input geometry arrays that belong to the specified relation. Both arrays are
	 * assumed to be in the same spatial reference. The relations are evaluated in 2D. Z coordinates are not used. Geometry types
	 * cannot be mixed within an array.
	 * 
	 * @param params - The set of parameters required to perform the comparison.
	 * @return Deferred
	 */
	public final native Deferred relation(RelationParameters params) /*-{		
		return this.relation(params);
	}-*/;
	
	/**
	 * The reshape operation is performed on a geometry service resource. It reshapes a polyline or a part of a polygon using a reshaping line. 
	 * 
	 * @param targetGeometry - The polyline or polygon to be reshaped.
	 * @param reshaperGeometry - The single-part polyline that does the reshaping.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred reshape(Geometry[] targetGeometry, Geometry reshaperGeometry, ReshapeCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < targetGeometry.length; i++ ) Util.addObjectToArray(targetGeometry[i], arr);
		return _reshape(arr, reshaperGeometry, callback);
	}
	
	private final native Deferred _reshape(JavaScriptObject targetGeometry, Geometry reshaperGeometry, ReshapeCallback callback) /*-{		
		return this.reshape(targetGeometry, reshaperGeometry, 
			function(geometry) {
				callback.@com.esri.gwt.client.callback.ReshapeCallback::onReshapeComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.ReshapeCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The reshape operation is performed on a geometry service resource. It reshapes a polyline or a part of a polygon using a reshaping line. 
	 * 
	 * @param targetGeometry - The polyline or polygon to be reshaped.
	 * @param reshaperGeometry - The single-part polyline that does the reshaping.
	 * @return Deferred
	 */
	public final Deferred reshape(Geometry[] targetGeometry, Geometry reshaperGeometry) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < targetGeometry.length; i++ ) Util.addObjectToArray(targetGeometry[i], arr);
		return _reshape(arr, reshaperGeometry);
	}
	
	private final native Deferred _reshape(JavaScriptObject targetGeometry, Geometry reshaperGeometry) /*-{		
		return this.reshape(targetGeometry, reshaperGeometry);
	}-*/;
	
	/**
	 * Alters the given geometries to make their definitions topologically legal with respect to their geometry type. On completion,
	 * the onSimplifyComplete event is fired and the optional callback function is invoked. Both the callback and event handlers
	 * receive an array of Geometry that contains the simplified geometries.
	 * 
	 * @param geometries - The geometries to simplify.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred simplify(Geometry[] geometries, SimplifyCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _simplify(arr, callback);
	}
	
	private final native Deferred _simplify(JavaScriptObject geometries, SimplifyCallback callback) /*-{		
		return this.simplify(geometries,
			function(geometries) {
				callback.@com.esri.gwt.client.callback.SimplifyCallback::onSimplifyComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.SimplifyCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Alters the given geometries to make their definitions topologically legal with respect to their geometry type. On completion,
	 * the onSimplifyComplete event is fired and the optional callback function is invoked. Both the callback and event handlers
	 * receive an array of Geometry that contains the simplified geometries.
	 * 
	 * @param geometries - The geometries to simplify.
	 * @return Deferred
	 */
	public final Deferred simplify(Geometry[] geometries) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _simplify(arr);
	}
	
	private final native Deferred _simplify(JavaScriptObject geometries) /*-{		
		return this.simplify(geometries);
	}-*/;
	
	/**
	 * Trims or extends the input polylines using the user specified guide polyline. When trimming features, the portion to the left
	 * of the cutting line is preserved in the output and the rest is discarded. If the input polyline is not cut or extended then an empty
	 * polyline is added to the output array.
	 * 
	 * @param params - Input parameters for the trimExtend operation.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred trimExtend(TrimExtendParameters params, TrimExtendCallback callback) /*-{		
		return this.trimExtend(params,
			function(geometries) {
				callback.@com.esri.gwt.client.callback.TrimExtendCallback::onTrimExtendComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.TrimExtendCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Trims or extends the input polylines using the user specified guide polyline. When trimming features, the portion to the left
	 * of the cutting line is preserved in the output and the rest is discarded. If the input polyline is not cut or extended then an empty
	 * polyline is added to the output array.
	 * 
	 * @param params - Input parameters for the trimExtend operation.
	 * @return Deferred
	 */
	public final native Deferred trimExtend(TrimExtendParameters params) /*-{		
		return this.trimExtend(params);
	}-*/;
	
	/**
	 * The union operation is performed on a geometry service resource. This operation constructs the set-theoretic union of the
	 * geometries in the input array. All inputs must be of the same type. 
	 * 
	 * @param geometries - The array of geometries to be unioned.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final Deferred union(Geometry[] geometries, UnionCallback callback) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _union(arr, callback);
	}
	
	private final native Deferred _union(JavaScriptObject geometries, UnionCallback callback) /*-{		
		return this.union(geometries,
			function(geometry) {
				callback.@com.esri.gwt.client.callback.UnionCallback::onUnionComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.UnionCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * The union operation is performed on a geometry service resource. This operation constructs the set-theoretic union of the
	 * geometries in the input array. All inputs must be of the same type. 
	 * 
	 * @param geometries - The array of geometries to be unioned.
	 * @return Deferred
	 */
	public final Deferred union(Geometry[] geometries) {
		JavaScriptObject arr = JavaScriptObject.createArray().cast();
		for( int i = 0; i < geometries.length; i++ ) Util.addObjectToArray(geometries[i], arr);
		return _union(arr);
	}
	
	private final native Deferred _union(JavaScriptObject geometries) /*-{		
		return this.union(geometries);
	}-*/;
	
	/**
	 * Add event handler for areasAndLengths
	 * 
	 * @param handler - Fires when the areasAndLengths operation is complete. 
	 */
	public final native void addAreasAndLengthsHandler(AreasAndLengthsHandler handler) /*-{
		$wnd.dojo.connect(this, "onAreasAndLengthsComplete",
			function(results) {
				handler.@com.esri.gwt.client.event.AreasAndLengthsHandler::onAreasAndLengthsComplete(Lcom/esri/gwt/client/event/AreasAndLengthsHandler$AreasAndLengths;)(results);
			}
		);
	}-*/;
	
	/**
	 * Add event handler for autoComplete
	 * 
	 * @param handler - Fires when the autoComplete operation is complete.
	 */
	public final native void addAutoCompleteHandler(AutoCompleteHandler handler) /*-{
			$wnd.dojo.connect(this, "onAutoCompleteComplete",
				function(polygons) {
					handler.@com.esri.gwt.client.event.AutoCompleteHandler::onAutoCompleteComplete(Lcom/google/gwt/core/client/JsArray;)(polygons);
				}
			);	
	}-*/;
	
	/**
	 * Add event handler for buffer.
	 * 
	 * @param handler - Fires when the buffer operation is complete.
	 */
	public final native void addBufferHandler(BufferHandler handler) /*-{
		$wnd.dojo.connect(this, "onBufferComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.BufferHandler::onBufferComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for convexHull
	 * 
	 * @param handler - Fires when the convexHull operation is complete.
	 */
	public final native void addConvexHullHandler(ConvexHullHandler handler) /*-{
		$wnd.dojo.connect(this, "onConvexHullComplete",
			function(geometry) {
				handler.@com.esri.gwt.client.event.ConvexHullHandler::onConvexHullComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for cut.
	 * 
	 * @param handler - Fires when the cut operation is complete.
	 */
	public final native void addCutHandler(CutHandler handler) /*-{
		$wnd.dojo.connect(this, "onCutComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.CutHandler::onCutComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for difference.
	 * 
	 * @param handler - Fires when the difference operation is complete. 
	 */
	public final native void addDifferenceHandler(DifferenceHandler handler) /*-{
		$wnd.dojo.connect(this, "onDifferenceComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.DifferenceHandler::onDifferenceComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for distance.
	 * 
	 * @param handler - Fires when the difference operation is complete.
	 */
	public final native void addDistanceHandler(DistanceHandler handler) /*-{
		$wnd.dojo.connect(this, "onDistanceComplete",
			function(distance) {
				handler.@com.esri.gwt.client.event.DistanceHandler::onDistanceComplete(F)(distance);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for errors.
	 * 
	 * @param handler - Fires when an error occurs when executing the task. 
	 */
	public final native void addErrorHandler(ErrorHandler handler) /*-{
		$wnd.dojo.connect(this, "onError",
			function(error) {
				handler.@com.esri.gwt.client.event.ErrorHandler::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for generalize.
	 * 
	 * @param handler - Fires when the generalize operation is complete.
	 */
	public final native void addGeneralizeHandler(GeneralizeHandler handler) /*-{
		$wnd.dojo.connect(this, "onGeneralizeComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.GeneralizeHandler::onGeneralizeComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for intersect.
	 * 
	 * @param handler - Fires when the intersect operation is complete.
	 */
	public final native void addIntersectHandler(IntersectHandler handler) /*-{
		$wnd.dojo.connect(this, "onIntersectComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.IntersectHandler::onIntersectComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for labelPoints.
	 * 
	 * @param handler - Fires when the labelPoints operation is complete.
	 */
	public final native void addLabelPointsHandler(LabelPointsHandler handler) /*-{
		$wnd.dojo.connect(this, "onLabelPointsComplete",
			function(labelPoints) {
				handler.@com.esri.gwt.client.event.LabelPointsHandler::onLabelPointsComplete(Lcom/google/gwt/core/client/JsArray;)(labelPoints);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for lengths.
	 * 
	 * @param handler - Fires when the lengths operation is complete. 
	 */
	public final native void addLengthsHandler(LengthsHandler handler) /*-{
		$wnd.dojo.connect(this, "onLengthsComplete",
			function(lengths) {
				handler.@com.esri.gwt.client.event.LengthsHandler::onLengthsComplete(Lcom/esri/gwt/client/event/LengthsHandler$Lengths;)(lengths);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for offset.
	 * 
	 * @param handler - Fires when the offset operation is complete.
	 */
	public final native void addOffsetHandler(OffsetHandler handler) /*-{
		$wnd.dojo.connect(this, "onOffsetComplete",
			function(offset) {
				handler.@com.esri.gwt.client.event.OffsetHandler::onOffsetComplete(Lcom/google/gwt/core/client/JsArray;)(offset);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for project.
	 * 
	 * @param handler - Fires when the project operation is complete.
	 */
	public final native void addProjectHandler(ProjectHandler handler) /*-{
		$wnd.dojo.connect(this, "onProjectComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.ProjectHandler::onProjectComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for relation.
	 * 
	 * @param handler - Fires when the relation operation is complete. 
	 */
	public final native void addRelationHandler(RelationHandler handler) /*-{
		$wnd.dojo.connect(this, "onRelationComplete",
			function(relationships) {
				handler.@com.esri.gwt.client.event.RelationHandler::onRelationComplete(Lcom/google/gwt/core/client/JsArray;)(relationships);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for reshape.
	 * 
	 * @param handler - Fires when the reshape operation is complete.
	 */
	public final native void addReshapeHandler(ReshapeHandler handler) /*-{
		$wnd.dojo.connect(this, "onReshapeComplete",
			function(geometry) {
				handler.@com.esri.gwt.client.event.ReshapeHandler::onReshapeComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for simplify
	 * 
	 * @param handler - Fires when the trimExtend operation is complete.
	 */
	public final native void addSimplifyHandler(SimplifyHandler handler) /*-{
		$wnd.dojo.connect(this, "onSimplifyComplete",
			function(geometries) {
				handler.@com.esri.gwt.client.event.SimplifyHandler::onSimplifyComplete(Lcom/google/gwt/core/client/JsArray;)(geometries);
			}
		);	
	}-*/;
	
	/**
	 * Add event handler for union.
	 * 
	 * @param handler - Fires when the union operation is complete.
	 */
	public final native void addUnionHandler(UnionHandler handler) /*-{
		$wnd.dojo.connect(this, "onUnionComplete",
			function(geometry) {
				handler.@com.esri.gwt.client.event.UnionHandler::onUnionComplete(Lcom/esri/gwt/client/geometry/Geometry;)(geometry);
			}
		);	
	}-*/;

	
}
