package com.esri.gwt.client.tasks;

import com.esri.gwt.client.callback.RouteSolveCallback;
import com.esri.gwt.client.dojo.Deferred;
import com.esri.gwt.client.event.ErrorHandler;
import com.esri.gwt.client.event.RouteSolveHandler;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * Solves a route on a route layer resource in a Network Analyst service exposed by the ArcGIS Server REST API.
 * 
 * @author Justin Merz
 */
public class RouteTask extends JavaScriptObject {
	
	protected RouteTask() {}
	
	/**
	 * Creates a new RouteTask object.
	 * 
	 * @param url - URL to the ArcGIS Server REST resource that represents a network analysis service.
	 * @return RouteTask
	 */
	public static native RouteTask create(String url) /*-{
		return new $wnd.esri.tasks.RouteTask(url);
	}-*/;
	
	/**
	 * URL to the ArcGIS Server REST resource that represents a network analysis service.
	 * 
	 * @return String url.
	 */
	public final native String getUrl() /*-{
		return this.url;
	}-*/;
	
	/**
	 * Solves the route against the route layer with the route parameters.
	 * 
	 * @param params - Route parameters used as input to generate the route.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred solve(RouteParameters params, RouteSolveCallback callback) /*-{		
		this.solve(params, 
			function(solveResult) {
				callback.@com.esri.gwt.client.callback.RouteSolveCallback::onSolveComplete(Lcom/esri/gwt/client/event/RouteSolveHandler$SolveResult;)(solveResult);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.RouteSolveCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Solves the route against the route layer with the route parameters.
	 * 
	 * @param params - Route parameters used as input to generate the route.
	 * @return Deferred
	 */
	public final native Deferred solve(RouteParameters params) /*-{		
		this.solve(params);
	}-*/;

	/**
	 * Add event handler for errors.
	 * 
	 * @param handler - Fires when an error occurs when executing the task.
	 */
	public final native void addErrorHandler(ErrorHandler handler) /*-{
		$wnd.dojo.connect(this,  "onError",
			function(error) {
				handler.@com.esri.gwt.client.event.ErrorHandler::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Add event handler for solve.
	 * 
	 * @param handler - Fires when RouteTask.solve() has completed.
	 */
	public final native void addSolveHandler(RouteSolveHandler handler) /*-{
		$wnd.dojo.connect(this,  "onSolveComplete",
			function(solveResult) {
				handler.@com.esri.gwt.client.event.RouteSolveHandler::onSolveComplete(Lcom/esri/gwt/client/event/RouteSolveHandler$SolveResult;)(solveResult);
			}
		);
	}-*/;
	
}
