package com.esri.gwt.client.tasks;

import com.esri.gwt.client.callback.ServiceAreaSolveCallback;
import com.esri.gwt.client.dojo.Deferred;
import com.esri.gwt.client.event.ServiceAreaSolveHandler;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * Helps you find service areas around any location on a network. A network service area is a region that
 * encompasses all accessible streets that is, streets that are within a specified impedance. For instance,
 * the 5-minute service area for a point includes all the streets that can be reached within five minutes
 * from that point.
 * 
 * Note: ServiceAreaTask, and other service area related classes, requires ArcGIS Server 10.0 or above
 * and a service area layer. A service area layer is a layer of type esriNAServerServiceAreaLayer.
 * 
 * @author Justin Merz
 */
public class ServiceAreaTask extends JavaScriptObject {
	
	protected ServiceAreaTask() {}
	
	/**
	 * Creates a new ServiceAreaTask object.
	 * 
	 * @param url - URL to the ArcGIS Server REST resource that represents a network analysis service. Use the Service Directory to obtain the URL.
	 * @return ServiceAreaTask
	 */
	public static native ServiceAreaTask create(String url) /*-{
		return new $wnd.esri.tasks.ServiceAreaTask(url);
	}-*/;
	
	/**
	 * Solve the service area. 
	 * 
	 * @param params - The ServiceAreaParameters object.
	 * @param callback - The function to call when the method has completed.
	 * @return Deferred
	 */
	public final native Deferred solve(ServiceAreaParameters params, ServiceAreaSolveCallback callback) /*-{
		return this.solve( params,
			function(serviceAreaSolveResult) {
				callback.@com.esri.gwt.client.callback.ServiceAreaSolveCallback::onSolveComplete(Lcom/esri/gwt/client/tasks/ServiceAreaSolveResult;)(serviceAreaSolveResult);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.ServiceAreaSolveCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Add event handler to listen to the onSolveComplete event 
	 * 
	 * @param handler - Fired when the ServiceAreaTask solve is complete.
	 */
	public final native void addSolveHandler(ServiceAreaSolveHandler handler) /*-{
		$wnd.dojo.connect(this, "onSolveComplete",
			function(serviceAreaSolveResult) {
				handler.@com.esri.gwt.client.event.ServiceAreaSolveHandler::onSolveComplete(Lcom/esri/gwt/client/tasks/ServiceAreaSolveResult;)(serviceAreaSolveResult);
			}
		);
	}-*/;

}
